import React, { Component } from 'react';

import Indexes from './components/watch/Indexes';
import Hand from './components/watch/Hand';
import Grid from './components/watch/Grid';
import Burst from './components/watch/dials/Burst';
import Iris from './components/watch/dials/Iris';

import IrisBg from 'react-backgrounds/bin/Iris.js';
import Butterflies from 'react-backgrounds/bin/Butterflies.js';
import RandomTriangles from 'react-backgrounds/bin/RandomTriangles.js';

class Design extends Component {
  constructor(props) {
    super(props);

    this.state = {
      caseClasses: {
        bg: '',
        display: 'd-block',
        padding: 'p-1',
        shape: ''
      },
      dialClasses: {
        bg: '',
        display: 'd-block',
        padding: 'p-1',
        shape: ''
      },
      watchHeight: null
    }
  }

  componentWillMount() {
    var styles = this.props.styles;
    if (styles && styles.case) {
      this.setCaseStyles(styles.case);
    }
    if (styles && styles.dial) {
      this.setDialStyles(styles.dial);
    }
  }

  componentDidMount() {
    var watch = this.watch;
    var watchWidth = watch.clientWidth;

    this.setState({
      watchHeight: watchWidth
    });

    window.addEventListener("resize", this.setWatchHeight.bind(this));
  }

  setWatchHeight() {
    var watch = this.watch;
    var watchWidth = watch.clientWidth;

    this.setState({
      watchHeight: watchWidth
    });
  }

  setCaseStyles(styles) {
    var bg;
    if (styles.bg) {
      bg = styles.bg;
    }

    var padding;
    if (styles.padding) {
      padding = styles.padding;
    } else {
      padding = this.state.caseClasses.padding;
    }

    var shape;
    if (styles.shape) {
      shape = styles.shape;
    }

    this.setState({
      caseClasses: Object.assign({}, this.state.caseClasses, {
        bg: bg,
        padding: padding,
        shape: shape
      }),
    });

  }

  setDialStyles(styles) {
    var bg;
    if (styles.bg) {
      bg = styles.bg;
    }

    var padding;
    if (styles.padding) {
      padding = styles.padding;
    } else {
      padding = this.state.dialClasses.padding;
    }

    var shape;
    if (styles.shape) {
      shape = styles.shape;
    }

    this.setState({
      dialClasses: Object.assign({}, this.state.dialClasses, {
        bg: bg,
        padding: padding,
        shape: shape
      }),
    });
  }

  render() {
    var styles = this.props.styles;

    var burst;
    if (styles && styles.burst) {
      burst = <Burst />;
    }

    var irisBg;
    if (styles && styles.irisBg) {
      var colors;
      if (styles.irisBg.colors) {
        colors = styles.irisBg.colors;
      } else {
        colors = [
          '#ffdd00',
          '#f39100',
          '#e3000f',
          '#b71a49',
          '#006b94',
          '#009bb4',
          '#94c11c',
          '#000000'];
      }
      irisBg = (
        <IrisBg
          shapeCount={3}
          colors={colors}
          opacity={.5} />
      );
    }

    var flare;
    if (styles && styles.flare) {
      var colors;
      if (styles.flare.colors) {
        colors = styles.flare.colors;
      } else {
        colors = [
          '#ffdd00',
          '#f39100',
          '#e3000f',
          '#b71a49',
          '#006b94',
          '#009bb4',
          '#94c11c',
          '#000000'];
      }
      flare = (
        <RandomTriangles
          shapeCount={66}
          colors={colors}
          opacity={.5} />
      );
    }

    var butterflies;
    if (styles && styles.butterflies) {
      var colors;
      if (styles.butterflies.colors) {
        colors = styles.butterflies.colors;
      } else {
        colors = [
          '#ffdd00',
          '#f39100',
          '#e3000f',
          '#b71a49',
          '#006b94',
          '#009bb4',
          '#94c11c',
          '#000000'];
      }
      butterflies = (
        <Butterflies
          shapeCount={33}
          colors={colors}
          opacity={.8} />
      );
    }

    var iris;
    if (styles && styles.iris) {
      iris = <Iris fluteCount={20} />;
    }

    var indexes;
    if (styles && styles.index) {
      var indexStyles = this.props.styles.index;
      indexes = <Indexes styles={indexStyles} />;
    }

    var hands = [];
    if (styles && styles.hands) {
      var handStyles = styles.hands;
      if (handStyles && handStyles.hour) {
        var hourHand = <Hand
          key='hour'
          styles={handStyles.hour}
          type={handStyles.type.hour}
          timeZone={styles.timeZone} />
        hands.push(hourHand);
      }
      if (handStyles && handStyles.minute) {
        var minuteHand = <Hand
          key='minute'
          styles={handStyles.minute}
          type={handStyles.type.minute} />
        hands.push(minuteHand);
      }
      if (handStyles && handStyles.second) {
        var secondHand = <Hand
          key='second'
          styles={handStyles.second}
          type={handStyles.type.second} />
        hands.push(secondHand);
      }
    }

    var grid;
    if (styles && styles.grid) {
      var gridStyles = styles.grid;
      grid = <Grid styles={gridStyles} />
    }

    var caseClasses = this.state.caseClasses;
    caseClasses = Object.values(caseClasses);
    caseClasses = caseClasses.join(' ');

    var dialClasses = this.state.dialClasses;
    dialClasses = Object.values(dialClasses);
    dialClasses = dialClasses.join(' ');

    return (
      <div
        ref={(watch) => { this.watch = watch; }}
        className={caseClasses}
        style={{
          height: this.state.watchHeight,
          border: 'solid black 1px'
        }}>
        <div
          className={dialClasses}
          style={{
            width: '100%',
            height: '100%',
            position: 'relative',
            border: 'solid black 1px',
            overflow: 'hidden'
          }}>
          {burst}
          {irisBg}
          {flare}
          {butterflies}
          {iris}
          {indexes}
          {grid}
          {hands}
        </div>
      </div>
    );
  }
}

export default Design;
